# !/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
# Time       ：2023/8/7 15:47
# Author     ：BeyondAllan
# Description：
"""
import sys
import requests
import urllib3


class HttpRequest:
    """
    实现http交互
    """
    # 目标服务器节点
    host = "10.1.100.121"
    # base url path
    base_url = f"https://{host}/api"
    # 请求数据格式
    accept = "application/json, text/plain, */*"
    # 连接方式
    connection = "keep-alive"

    def __init__(self, username=None, password=None):
        """
        初始化请求
        :param username: 用于获取认证token
        :param password: 用于获取认证token
        """
        self.__login__(username, password)

    def __login__(self, username, password):
        """
        登录获取请求认证
        :return:
        """
        headers = {
            "Accept": self.accept,
            "Authorization": "",
            "Connection": self.connection,
            "Host": self.host,
            # Add more headers if needed
        }
        url = f"{self.base_url}/v1/login"
        data = {
            "name": username,
            "password": password
        }

        response = requests.post(url, headers=headers, json=data, verify=False)

        if response.status_code == 200:
            response_data = response.json()
            token = response_data.get("access_token")

            # 本次登录认证密钥
            self.authorization = f"bearer {token}"

            # 构造默认的请求认证、请求头
            self.headers = {
                "Accept": self.accept,
                "Authorization": self.authorization,
                "Connection": self.connection,
                "Host": self.host,
                # Add more headers if needed
            }

        else:
            raise TaskError(f"url:{url} request failed with status code: {response.status_code}")

    def get_all_servers_details(self):
        """
        获取所有的服务器详情信息，包含硬盘信息
        :return:
        """
        headers = self.headers
        # url参数
        params = {
            "action": "create"
        }

        url = f"{self.base_url}/v2/failure_domain/disk"

        response = requests.get(url, params=params, headers=headers, verify=False)
        if response.status_code == 200:
            return response.json()
        else:
            raise TaskError(f"url:{url} request failed with status code: {response.status_code}")

    def get_tianshu_cluster(self):
        """
        获取天枢集群
        :return:
        """
        headers = self.headers
        # url参数
        params = {}

        url = f"{self.base_url}/v2/tianshu"

        response = requests.get(url, params=params, headers=headers, verify=False)
        if response.status_code == 200:
            result = response.json()
            return result.get("result")
        else:
            raise TaskError(f"url:{url} equest failed with status code: {response.status_code}")

    def create_pool(self, pool_data):
        """
        创建极速池
        :param pool_data:
        :return:
        """
        headers = self.headers
        # url参数
        data = pool_data

        url = f"{self.base_url}/v2/failure_domain"

        response = requests.post(url, headers=headers, json=data, verify=False)
        if response.status_code == 200:
            result = response.json()
            return result.get("task_id")
        else:
            raise TaskError(f"url:{url} request failed with status code: {response.status_code}")

    def get_all_uss_servers(self):
        """
        获取所有的uss角色服务器详情
        :return:
        """
        headers = self.headers
        # url参数
        params = {}

        url = f"{self.base_url}/v2/wds/uss/servers"

        response = requests.get(url, params=params, headers=headers, verify=False)
        if response.status_code == 200:
            result = response.json()
            return result.get("uss_servers")
        else:
            raise TaskError(f"url request failed with status code: {response.status_code}")

    def create_uss_gateway(self, uss_data):
        """

        :param uss_data:
        :return:
        """
        headers = self.headers
        # url参数
        data = uss_data

        url = f"{self.base_url}/v2/wds/uss"

        response = requests.post(url, headers=headers, json=data, verify=False)
        if response.status_code == 200:
            result = response.json()
            return result.get("task_id")
        else:
            raise TaskError(f"url:{url} request failed with status code: {response.status_code}")


class HyperPool:
    """
    极速池类，包含对极速池的操作
    """
    def __init__(self, tianshu_id):
        # TODO：目前除必要实时获取参数对外，其他参数默认
        self.failure_domain_name = "HyperPool"
        self.safe_level = "host"
        self.physical_pool_type = "local_pool"
        self.osd_network = "RDMA"
        self.redundancy_ploy = "replicated"
        self.tianshu_id = tianshu_id
        self.perf_mode = "high"
        self.replicate_size = 2
        self.cache_meta_data = False
        self.db_cache_size = 0
        self.cache_data = False

    def config_pool_create_params(self, servers_detail, name):
        """
        从服务器详情数据构建创建极速池的参数与数据
        :param name:
        :param servers_detail:
        :return:
        """
        create_params = {
            "failure_domain_name": name,
            "safe_level": self.safe_level,
            "domain_type": "",
            "cache_disk_type": None,
            "osd_count": 0,
            "osd_members": [],
            "physical_pool_type": self.physical_pool_type,
            "cache_meta_data": self.cache_meta_data,
            "db_cache_size": self.db_cache_size,
            "cache_data": self.cache_data,
            "osd_network": self.osd_network,
            "redundancy_ploy": self.redundancy_ploy,
            "tianshu_id": self.tianshu_id,
            "perf_mode": self.perf_mode,
            "replicate_size": self.replicate_size
        }

        osd_members = []
        count = 0
        for rack in servers_detail.get("racks"):
            servers = rack.get('servers')

            for server in servers:
                # 控制每个极速池4个服务器
                if count == 4:
                    continue
                member = {}
                roles = server.get("roles")
                if "wds_storage" not in roles:
                    continue
                member["rack_name"] = rack.get("rack")
                member["server_name"] = server.get("name")
                member["server_id"] = server.get("id")
                member["server_type"] = server.get("server_type")
                member["datacenter"] = rack.get("datacenter")
                member["cache_disk_list"] = []

                disks = server.get("disks")
                osds = []
                for disk in disks:
                    if disk.get("osd") or disk.get("disk_partitions") or disk.get("is_os_disk"):
                        continue
                    size = disk.get("size")
                    disk_id = disk.get("id")
                    protocol_type = disk.get("protocol_type")
                    media_type = disk.get("media_type")
                    create_params["domain_type"] = f"{protocol_type}_{media_type}"
                    osds.append({
                        "name": "",
                        "size": size,
                        "disk_id": disk_id,
                        "flag": "create"
                    })
                if not osds:
                    continue
                member["osds"] = osds
                create_params["osd_count"] += len(osds)
                osd_members.append(member)
                count += 1
        create_params["osd_members"] = osd_members
        if not osd_members:
            return None

        return create_params


class USS:
    """
    USS类，包含对USS的操作
    """
    def __init__(self, tianshu_id, server_name, interface_ids):
        # TODO：目前除必要实时获取参数对外，其他参数默认
        self.type = "uss"
        self.server_name = server_name
        self.tianshu_id = tianshu_id
        self.protocol = "NVMe-oF"
        self.specification = "standard"
        self.interface_ids = interface_ids
        self.uss_network = "RDMA"
        self.protocol_network = "RDMA"

    def config_uss_create_params(self, num="1"):
        """
        创建USS参数
        :return:
        """
        create_params = {
            "uss_gateways": [
                {
                    "type": self.type,
                    "name": f"{self.server_name}_gw_{num}",
                    "Tianshu_id": self.tianshu_id,
                    "protocol": self.protocol,
                    "specification": self.specification,
                    "uss_network": self.uss_network,
                    "protocol_network": self.protocol_network,
                    "interface_ids": self.interface_ids
                }
            ]
        }

        return create_params


class TaskError(Exception):
    """
    有异常时，通过该异常捕获退出
    """
    def __init__(self, message=""):
        super().__init__(message)


def help_template():
    """
    帮助菜单
    :return:
    """
    template = """
    ****************************************************************************
    该脚本用于自动化创建极速池或者uss资源
    使用说明：
    1. 当创建资源为极速池时：
    参数1：username (登录管理平台的用户名)
    参数2：password (对应用户的登录密码)
    参数3：type (本次创建的类型：[pool/uss] )
    参数4：pool_name (极速池名称前缀)
    
    示例： python3 auto_create.py operator Admin123 pool pool
    
    2. 当创建资源为USS网关时：
    参数1：username (登录管理平台的用户名)
    参数2：password (对应用户的登录密码)
    参数3：type (本次创建的类型：[pool/uss] )
    参数4：server_names (预期创建uss的计算服务器名称列表, 多个时通过【,】分隔
            注意：可填写为 all 此时将全部uss计算服务器创建, 每个计算服务器默认创建4个)
    
    示例： 
        1. python3 auto_create.py operator Admin123 uss node189,node190
        2. python3 auto_create.py operator Admin123 uss all

    ****************************************************************************
    """
    return template


# 工具主服务
def main():
    """

    :return:
    """
    arguments = sys.argv[1:]

    if "-h" in arguments:
        print(help_template())
        return

    username = arguments[0]
    password = arguments[1]
    resource_type = arguments[2]

    user = username
    password = password

    print("#"*80)
    print("1. log in ...")
    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
    try:
        http_tool = HttpRequest(username=user, password=password)
        # 获取天枢集群id
        tianshu_infos = http_tool.get_tianshu_cluster()
        tianshu_id = tianshu_infos[0].get("id")
        print(f"2. get ts node ID: [{tianshu_id}]")
        print("#" * 80)
    except Exception as ex:
        print(f"faild：{ex}")
        print("exit")
        return

    # 极速池
    try:
        if resource_type == "pool":
            pool_name = arguments[3]

            # 创建8个极速池
            for num in range(1, 9):
                print(f"-- create pool: [{pool_name}-{num}]")
                # 获取集群所有服务器以及硬盘信息
                servers_detail = http_tool.get_all_servers_details()
                # 创建极速池
                # print(f"-- create pool: [f{pool_name}-{num}] ")
                create_data = HyperPool(tianshu_id=tianshu_id).config_pool_create_params(servers_detail=servers_detail,
                                                                                         name=f"{pool_name}-{num}")
                if not create_data:
                    raise TaskError("无法获取符合创建条件的物理资源，请检查")
                task_id = http_tool.create_pool(create_data)
                #print(f"-- 极速池: [{pool_name}-{num}] 创建任务 [{task_id}] 已下发, 等待并查看创建结果")

        # uss网关
        elif resource_type == "uss":
            print("3.start create USS")
            create_servers = arguments[3]

            uss_servers = http_tool.get_all_uss_servers()
            server_interfaces = {server.get("name"): [net.get("interface_id")] for server in uss_servers for net in
                                 server.get("business_net")}
            if not server_interfaces:
                raise TaskError("无法获取符合创建条件的物理资源，请检查")
            create_servers = server_interfaces.keys() if create_servers == "all" else create_servers.split(",")

            for num in ["01", "02", "03", "04"]:
                for server_name in create_servers:
                    # 创建USS
                    print(f"-- USS: [{server_name}_gw_{num}]")
                    create_data = USS(tianshu_id=tianshu_id, server_name=server_name,
                                      interface_ids=server_interfaces.get(
                                          server_name)).config_uss_create_params(num=num)
                    if not create_data:
                        raise TaskError("无法获取符合创建条件的物理资源，请检查")
                    print(f"{create_data}")
                    task_id = http_tool.create_uss_gateway(create_data)
                    #print(f"-- USS网关: [{server_name}_gw_{num}] 创建任务 [{task_id}] 已下发, 等待并查看创建结果")
                    print("*" * 40)
                else:
                    print(f"exit")
    except Exception as ex:
        print(f"faild：{ex}")
        print("exit")
        return

    print(f"exit...")


# 服务启动位置
if __name__ == "__main__":
    main()
